"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Connection_1 = require("./dynamo/Connection");
const restify_errors_1 = require("restify-errors");
const config = require("config");
const chunk = require("lodash.chunk");
const flatten = require("lodash.flatten");
class DynamoDataSource {
    constructor() {
        this.table = config.get("aws.dynamodb.tableName");
        this.keyName = config.get("aws.dynamodb.keyName");
    }
    getData(key, fields) {
        const params = {
            TableName: this.table,
            Key: {}
        };
        params.Key[this.keyName] = key;
        if (!!fields) {
            params["ProjectionExpression"] = fields;
        }
        return new Promise((resolve, reject) => {
            Connection_1.Connection.getInstance().get(params, (err, data) => {
                if (err) {
                    reject(new restify_errors_1.InternalServerError("Unable to get item, error: " + err.message));
                }
                else {
                    resolve(data.Item);
                }
            });
        });
    }
    putData(key, value) {
        let params = {
            TableName: this.table,
            Item: value
        };
        return new Promise((resolve, reject) => {
            Connection_1.Connection.getInstance().put(params, (err, data) => {
                if (err) {
                    reject(new restify_errors_1.InternalServerError("Unable to create item, error: " + err.message));
                }
                else {
                    let response = {};
                    response[this.keyName] = key;
                    resolve(response);
                }
            });
        });
    }
    getItems(keys, fields) {
        return new Promise((resolveItems, rejectItems) => {
            const chunks = chunk(keys, 100);
            const chunksLength = chunks.length;
            const promises = [];
            for (let i = 0; i < chunksLength; i++) {
                const params = {
                    RequestItems: {
                        [this.table]: {
                            Keys: chunks[i].map(key => ({ [this.keyName]: key }))
                        }
                    }
                };
                if (!!fields) {
                    params["RequestItems"][this.table]["ProjectionExpression"] = fields;
                }
                promises.push(new Promise((resolve, reject) => {
                    Connection_1.Connection.getInstance().batchGet(params, (err, data) => {
                        if (err) {
                            rejectItems(new restify_errors_1.InternalServerError("Unable to get items, error: " + err.message));
                        }
                        else {
                            const ordered = keys.map(key => data.Responses[this.table].find(row => row[this.keyName] === key)).filter(key => key);
                            resolve(ordered);
                        }
                    });
                }));
            }
            Promise.all(promises).then(values => {
                resolveItems(flatten(values));
            });
        });
    }
    updateData(key, value) {
        return new Promise((resolve, reject) => {
            this.getData(key).then((item) => {
                if (!!item) {
                    this.putData(key, value).then(function (data) {
                        resolve(data);
                    });
                }
                else {
                    reject(new restify_errors_1.NotFoundError("Item not found"));
                }
            }).catch((err) => {
                reject(new restify_errors_1.InternalServerError("Item not found, error", err));
            });
        });
    }
    searchData(query) {
        return new Promise((resolve, reject) => resolve(null));
    }
    deleteItem(key) {
        const params = {
            TableName: this.table,
            Key: {}
        };
        params.Key[this.keyName] = key;
        return new Promise((resolve, reject) => {
            Connection_1.Connection.getInstance().delete(params, (err, data) => {
                if (err)
                    reject(new restify_errors_1.NotFoundError(err));
                else
                    resolve(data);
            });
        });
    }
}
exports.DynamoDataSource = DynamoDataSource;
//# sourceMappingURL=DynamoDataSource.js.map