"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const restify_errors_1 = require("restify-errors");
const restler = require("restler");
const config = require("config");
const LoggerHelper_1 = require("../helper/logger/LoggerHelper");
/**
 * Class that represents an external data source (fallback to fetch external data)
 */
class ExternalDataSource {
    constructor(urlBuilder) {
        this.urlBuilder = urlBuilder;
    }
    getData(key, fields) {
        return new Promise((resolve, reject) => {
            try {
                let path = this.urlBuilder.getResourceUrlOrThrow(key);
                if (!!path) {
                    LoggerHelper_1.LoggerHelper.getDefaultHandler().info(`Request item from ${path}`);
                    restler.get(path, { timeout: config.get("server.options.timeout") })
                        .on("timeout", () => {
                        LoggerHelper_1.LoggerHelper.getDefaultHandler().error(`ExternalDataSource timeout`);
                        reject(new restify_errors_1.ServiceUnavailableError("External data source not respond"));
                    })
                        .on("fail", (err, response) => {
                        LoggerHelper_1.LoggerHelper.getDefaultHandler().error(`ExternalDataSource fail: ${err}`);
                        reject(new restify_errors_1.BadRequestError(err));
                    })
                        .on("error", (err, response) => {
                        LoggerHelper_1.LoggerHelper.getDefaultHandler().error(`ExternalDataSource error: ${err}`);
                        reject(new restify_errors_1.ServiceUnavailableError(err));
                    })
                        .on("success", (result) => {
                        resolve(result);
                    });
                }
                else {
                    reject(new restify_errors_1.NotAcceptableError("The source " + key + " can't be parsed."));
                }
            }
            catch (e) {
                reject(new Error(e.message));
            }
        });
    }
    putData(key, value) {
        return new Promise((resolve) => {
            resolve(true);
        });
    }
    getItems(keys, fields) {
        return new Promise((resolve, reject) => {
            try {
                let path = this.urlBuilder.getMultiGetResourceUrl(keys);
                if (!!path) {
                    LoggerHelper_1.LoggerHelper.getDefaultHandler().info(`Request items from ${path}`);
                    restler.get(path, { timeout: config.get("server.options.timeout") })
                        .on("timeout", () => {
                        LoggerHelper_1.LoggerHelper.getDefaultHandler().error(`ExternalDataSource timeout`);
                        reject(new restify_errors_1.ServiceUnavailableError("External data source not respond"));
                    })
                        .on("fail", (err, response) => {
                        LoggerHelper_1.LoggerHelper.getDefaultHandler().error(`ExternalDataSource fail: ${err}`);
                        reject(new restify_errors_1.BadRequestError(err));
                    })
                        .on("error", (err, response) => {
                        LoggerHelper_1.LoggerHelper.getDefaultHandler().error(`ExternalDataSource: ${err}`);
                        reject(new restify_errors_1.ServiceUnavailableError(err));
                    })
                        .on("success", (result) => {
                        resolve(result);
                    });
                }
                else {
                    reject(new restify_errors_1.NotAcceptableError("The sources " + keys + " can't be parsed."));
                }
            }
            catch (e) {
                reject(new Error(e.message));
            }
        });
    }
    updateData(key, value) {
        return new Promise((resolve, reject) => {
            reject(new restify_errors_1.InternalServerError("Can not update item to an external datasource"));
        });
    }
    searchData(query) {
        return new Promise((resolve, reject) => resolve(null));
    }
    deleteItem(key) {
        return Promise.reject(new Error('Method not implemented'));
    }
}
exports.ExternalDataSource = ExternalDataSource;
//# sourceMappingURL=ExternalDataSource.js.map