"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const restify_errors_1 = require("restify-errors");
const LoggerHelper_1 = require("../../helper/logger/LoggerHelper");
class DataSourceManager {
    constructor(dataSource, ...dataSources) {
        this.dataSources = [];
        this.dataSources.push(dataSource);
        if (!!dataSources) {
            dataSources.forEach((value, index, array) => {
                this.dataSources.push(value);
            });
        }
    }
    getData(key, fields) {
        return this.internal_get_data(0, key, fields);
    }
    putData(key, value) {
        return new Promise((resolve, reject) => {
            const dataSource = this.dataSources[0];
            if (!!dataSource) {
                dataSource.putData(key, value).then((data) => {
                    if (!!data) {
                        resolve(data);
                    }
                }).catch((err) => {
                    LoggerHelper_1.LoggerHelper.getDefaultHandler().error(err.message);
                    reject(new restify_errors_1.InternalServerError(err.message));
                });
            }
            else {
                LoggerHelper_1.LoggerHelper.getDefaultHandler().error("DataSource not found");
                reject(new restify_errors_1.ServiceUnavailableError("DataSource not found"));
            }
        });
    }
    getItems(keys, fields) {
        return this.internal_get_list(0, keys, fields);
    }
    updateData(key, value) {
        return new Promise((resolve, reject) => {
            const dataSource = this.dataSources[0];
            dataSource.updateData(key, value).then((data) => {
                if (!!data) {
                    resolve(data);
                }
            }).catch((err) => {
                LoggerHelper_1.LoggerHelper.getDefaultHandler().error(err.message);
                reject(err);
            });
        });
    }
    internal_get_data(index, key, fields) {
        let dataSource = this.dataSources[index];
        return new Promise((resolve, reject) => {
            if (!!dataSource) {
                dataSource.getData(key, fields).then((data) => {
                    if (!!data) {
                        if (index > 0) {
                            // update the main source
                            this.dataSources[0].putData(key, data);
                        }
                        resolve(data);
                    }
                    else {
                        // if the main source doesn't have data, update it with the next source
                        index++;
                        this.internal_get_data(index, key, fields).then((data) => {
                            resolve(data);
                        }).catch((err) => {
                            LoggerHelper_1.LoggerHelper.getDefaultHandler().error(err.message);
                            reject(err);
                        });
                    }
                }).catch((err) => {
                    LoggerHelper_1.LoggerHelper.getDefaultHandler().error(err.message);
                    reject(err);
                });
            }
            else {
                reject(new restify_errors_1.ResourceNotFoundError("Resource not found"));
            }
        });
    }
    internal_get_list(index, keys, fields, items) {
        let dataSource = this.dataSources[index];
        return new Promise((resolve, reject) => {
            if (!!dataSource) {
                dataSource.getItems(keys, fields).then((data) => {
                    // if we dont find all the keys we fetch the next datasource
                    if (!!data && data.length === keys.length) {
                        if (index > 0) {
                            // update the main source: we have to verify all the items and update which ones that dont exist
                            this.updateMainDatasource(keys, data, reject);
                        }
                        resolve(data);
                    }
                    else {
                        index++;
                        this.internal_get_list(index, keys, fields, data).then((data) => {
                            resolve(data);
                        }).catch((err) => {
                            LoggerHelper_1.LoggerHelper.getDefaultHandler().error(err.message);
                            reject(err);
                        });
                    }
                }).catch((err) => {
                    if (!!items) {
                        LoggerHelper_1.LoggerHelper.getDefaultHandler().warn(`Some items fail from: ${keys}`);
                        resolve(items);
                    }
                    else {
                        LoggerHelper_1.LoggerHelper.getDefaultHandler().error(err.message);
                        reject(err);
                    }
                });
            }
            else {
                reject(new restify_errors_1.ResourceNotFoundError("Resource not found"));
            }
        });
    }
    updateMainDatasource(keys, data, reject) {
        let dataIndex = 0;
        keys.map((key) => {
            if (!!data[dataIndex]) {
                this.dataSources[0].putData(key, data[dataIndex])
                    .catch((err) => {
                    // item not found
                    reject(err);
                });
            }
            dataIndex++;
        });
    }
    searchData(query) {
        return this.internal_search_data(0, query);
    }
    internal_search_data(index, query) {
        let dataSource = this.dataSources[index];
        return new Promise((resolve, reject) => {
            if (!!dataSource) {
                dataSource.searchData(query).then((data) => {
                    if (!!data) {
                        resolve(data);
                    }
                    else {
                        index++;
                        // try to search in the next datasource
                        this.internal_search_data(index, query).then((data) => {
                            resolve(data);
                        }).catch((err) => {
                            LoggerHelper_1.LoggerHelper.getDefaultHandler().error(err.message);
                            reject(err);
                        });
                    }
                }).catch((err) => {
                    LoggerHelper_1.LoggerHelper.getDefaultHandler().error(err.message);
                    reject(err);
                });
            }
            else {
                reject(new restify_errors_1.ResourceNotFoundError("Resource not found"));
            }
        });
    }
    deleteItem(key) {
        return new Promise((resolve, reject) => {
            const dataSource = this.dataSources[0];
            dataSource.deleteItem(key).then((data) => {
                if (!!data) {
                    resolve(data);
                }
            }).catch((err) => {
                LoggerHelper_1.LoggerHelper.getDefaultHandler().error(err.message);
                reject(err);
            });
        });
    }
}
exports.DataSourceManager = DataSourceManager;
//# sourceMappingURL=DataSourceManager.js.map