"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const restify = require("restify");
const ServerCharset_1 = require("./ServerCharset");
const config = require("config");
const corsMiddleware = require("restify-cors-middleware");
const yn = require("yn");
const queryParser = restify.plugins.queryParser({
    mapParams: true
});
const bodyParser = restify.plugins.bodyParser({
    mapParams: true
});
class ServerBuilder {
    constructor(routerConfigBuilder) {
        this._serverOptions = {};
        this._routerConfig = [];
        this._routerConfigBuilder = routerConfigBuilder;
        this._sanitizer = null;
        this._queryParser = null;
        this._charset = null;
        this._bodyParser = null;
        this._timeout = null;
        this._cors = true;
        this._middlewares = [];
        this._uncaughtExceptionHandler = null;
    }
    withOptions(serverOptions) {
        this._serverOptions = serverOptions;
        return this;
    }
    withRouter(routerConfig) {
        this._routerConfig.push(routerConfig);
        return this;
    }
    withRouterList(routerList) {
        routerList.map(router => this._routerConfig.push(router));
        return this;
    }
    withSanitizer(sanitizer) {
        this._sanitizer = sanitizer;
        return this;
    }
    withQueryParser(queryParser) {
        console.warn("restify's queryParser is designed to retrieve data from a url's query string. If you " +
            "are attempting to read post data from the request's body, please use the bodyParser plugin and the " +
            "method withBodyParser instead.");
        this._queryParser = queryParser;
        return this;
    }
    withCharset(charset) {
        this._charset = charset;
        return this;
    }
    withBodyParser(bodyParser) {
        this._bodyParser = bodyParser;
        return this;
    }
    withTimeout(timeout) {
        this._timeout = timeout;
        return this;
    }
    withCORS(enable) {
        this._cors = enable;
        return this;
    }
    withMiddleWare(component) {
        this._middlewares.push(component);
        return this;
    }
    withUncaughtExceptionHandler(handler) {
        this._uncaughtExceptionHandler = handler;
        return this;
    }
    withBugsnag() {
        if (config.get("bugsnag.enable")) {
            const bugsnag = require("bugsnag");
            bugsnag.register(config.get("bugsnag.api_key"), {
                releaseStage: config.get("bugsnag.release_stage")
            });
            this.withUncaughtExceptionHandler(bugsnag.restifyHandler);
        }
        return this;
    }
    withMorgan() {
        const morgan = require("morgan");
        this.withMiddleWare(morgan(config.get("morgan.format")));
        return this;
    }
    build() {
        let server = restify.createServer(this._serverOptions);
        if (!!this._bodyParser) {
            server.use(this._bodyParser);
        }
        else {
            server.use(bodyParser);
        }
        if (!!this._charset) {
            server.pre(this._charset);
        }
        else {
            server.pre(ServerCharset_1.default);
        }
        if (!!this._queryParser) {
            server.use(this._queryParser);
        }
        else {
            server.use(queryParser);
        }
        if (!!this._sanitizer) {
            server.pre(this._sanitizer);
        }
        else {
            server.pre(restify.pre.sanitizePath());
        }
        if (!!this._timeout) {
            let timeout = this._timeout;
            server.use(function (req, res, next) {
                req.connection.setTimeout(timeout);
                next();
            });
        }
        if (this._cors) {
            const cors = corsMiddleware({ "origins": ["*"], allowHeaders: ['authorization'] });
            server.pre(cors.preflight);
            server.use(cors.actual);
        }
        if (this._middlewares.length > 0) {
            for (const _middleware in this._middlewares) {
                if (this._middlewares.hasOwnProperty(_middleware)) {
                    server.use(this._middlewares[_middleware]);
                }
            }
        }
        if (this._uncaughtExceptionHandler) {
            server.on("uncaughtException", this._uncaughtExceptionHandler);
        }
        this._routerConfig.forEach((config) => {
            config.apply(this._routerConfigBuilder);
        });
        this._routerConfigBuilder.apply(server);
        server.pre(function (req, res, next) {
            if (!req.headers["accept-version"]) {
                req.headers["accept-version"] = config.get("server.options.defaultApiVersion");
            }
            next();
        });
        return server;
    }
}
exports.ServerBuilder = ServerBuilder;
//# sourceMappingURL=ServerBuilder.js.map